#!/usr/bin/python3

#
# Copyright (C) 2022 Jonathan Certes
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program. If not, see <http://www.gnu.org/licenses/>.
#

import sys
import pynusmv

#===============================================================================

##
# \brief
#   Creates a list of NuSMV node pointers that all share the same type
#   attribute.
#
# \details
#   Recursively accesses NuSMV nodes from one specific node and compares their
#   type: if it matches the expected type, adds them to the list that is given
#   as a parameter. Stops recursively accessing nodes once the type cannot have
#   a hierarchy.
#
# \returns void
#
def recursivelyGetAllNodes(
  theTopNode,   #!< the node to get in the hierarchy from
  theNodeType,  #!< the type of nodes to add to the list
  theList       #!< the list to append
):
  if ( theTopNode.type == theNodeType ):
    list.append(theList, theTopNode)
  else:
    if theTopNode.type not in {
      pynusmv.nusmv.parser.parser.NUMBER_SIGNED_WORD,
      pynusmv.nusmv.parser.parser.NUMBER_UNSIGNED_WORD,
      pynusmv.nusmv.parser.parser.NUMBER_FRAC,
      pynusmv.nusmv.parser.parser.NUMBER_EXP,
      pynusmv.nusmv.parser.parser.NUMBER_REAL,
      pynusmv.nusmv.parser.parser.NUMBER,
      pynusmv.nusmv.parser.parser.ATOM,
      pynusmv.nusmv.parser.parser.FAILURE
    }:
      if ( pynusmv.nusmv.node.node.car(theTopNode) is not None ):
        recursivelyGetAllNodes( pynusmv.nusmv.node.node.car(theTopNode),
                                theNodeType,
                                theList
                              )
      if ( pynusmv.nusmv.node.node.cdr(theTopNode) is not None ):
        recursivelyGetAllNodes( pynusmv.nusmv.node.node.cdr(theTopNode),
                                theNodeType,
                                theList
                              )

#===============================================================================

##
# \brief
#   Prints the content of specifications from a SMV file to the standard output.
#
# \returns void
#
def specnames_parseSmvFile(
  theFilePath #!< path to the input file containing PSL expressions
):
  pynusmv.init.reset_nusmv()
  # causes a segmentation fault if the input file is not correctly written
  # (catching the error cannot avoid NuSMV API to stop):
  pynusmv.nusmv.parser.parser.Parser_ReadSMVFromFile(str(theFilePath))

  # accesses all specification properties in the file:
  theTree = pynusmv.nusmv.parser.parser.cvar.parsed_tree
  theList = list()
  recursivelyGetAllNodes(theTree, pynusmv.nusmv.parser.parser.PSLSPEC,  theList)
  recursivelyGetAllNodes(theTree, pynusmv.nusmv.parser.parser.LTLSPEC,  theList)
  recursivelyGetAllNodes(theTree, pynusmv.nusmv.parser.parser.INVARSPEC,theList)
  theList.reverse()

  for theSpecNode in theList:
    # saves the name of the property (left node):
    theNameNode   = pynusmv.nusmv.node.node.cdr(theSpecNode)
    theNameString = pynusmv.nusmv.node.node.sprint_node(theNameNode)
    # prints it:
    if ( str.strip(theNameString) != "" ):
      print(theNameString)
      print("")

#===============================================================================

if ( len(sys.argv) < 2 ):
  print("** Error ** you must specify at least one argument")
  exit(1)

pynusmv.init.init_nusmv()
for theArg in sys.argv[1:len(sys.argv)]:
  specnames_parseSmvFile(theArg)

